import { ISerializeIterable, ISerializeObject, ISerializer, serialize, SerializeObject } from "serde/ser"

class JSONObjectSerializer extends SerializeObject<void> {
  private ser: JSONSerializer
  private first: boolean = true

  constructor(serializer: JSONSerializer) {
    super()
    this.ser = serializer
    serializer.write('{')
  }

  serializeKey<U>(key: U): void {
    if (!this.first) {
      this.ser.write(',')
    } else {
      this.first = false
    }

    serialize(this.ser, key)
    this.ser.write(':')
  }

  serializeValue<U>(value: U): void {
    serialize(this.ser, value)
  }

  end(): void {
    this.ser.write('}')
  }
}

class JSONIterableSerializer implements ISerializeIterable<void> {
  private ser: JSONSerializer
  private first: boolean = true

  constructor(serializer: JSONSerializer) {
    this.ser = serializer
    serializer.write('[')
  }

  serializeElement<U>(element: U): void {
    if (!this.first) {
      this.ser.write(',')
    } else {
      this.first = false
    }

    serialize(this.ser, element)
  }

  end(): void {
    this.ser.write(']')
  }
}

export class JSONSerializer implements ISerializer<void> {
  output: string = ''

  write(value: string) {
    this.output += value
  }

  serializeString(value: string) {
    this.write(`"${value}"`)
  }

  serializeBoolean(value: boolean): void {
    this.write(value.toString())
  }

  serializeSymbol(value: symbol): void {
    const key = Symbol.keyFor(value)
    if (key) {
      this.write(key)
    } else {
      return this.serializeString(value.toString())
    }
  }

  serializeObject(): ISerializeObject<void> {
    return new JSONObjectSerializer(this)
  }

  serializeClass(_name: PropertyKey): ISerializeObject<void> {
    return this.serializeObject()
  }

  serializeNumber(value: number) {
    this.write(value.toString())
  }

  serializeBigInt(value: bigint) {
    this.write(value.toString())
  }

  serializeIterable(): ISerializeIterable<void> {
    return new JSONIterableSerializer(this)
  }

  serializeNull() {
    return this.write('null')
  }
}


